"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RetrieveResultFormatter = void 0;
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
const path_1 = require("path");
const chalk_1 = require("chalk");
const core_1 = require("@salesforce/core");
const ts_types_1 = require("@salesforce/ts-types");
const source_deploy_retrieve_1 = require("@salesforce/source-deploy-retrieve");
const resultFormatter_1 = require("../resultFormatter");
core_1.Messages.importMessagesDirectory(__dirname);
const messages = core_1.Messages.loadMessages('@salesforce/plugin-source', 'retrieve');
class RetrieveResultFormatter extends resultFormatter_1.ResultFormatter {
    constructor(logger, ux, options, result) {
        var _a, _b, _c;
        var _d;
        super(logger, ux, options);
        // zipFile can become massive and unwieldy with JSON parsing/terminal output and, isn't useful
        delete result.response.zipFile;
        (_a = (_d = this.options).zipFileName) !== null && _a !== void 0 ? _a : (_d.zipFileName = 'unpackaged.zip');
        const zipFilePath = (0, path_1.join)(options.retrieveTargetDir, options.zipFileName);
        this.result = Object.assign({}, result.response, { zipFilePath });
        // grab warnings
        this.warnings = (0, resultFormatter_1.toArray)((_c = (_b = result === null || result === void 0 ? void 0 : result.response) === null || _b === void 0 ? void 0 : _b.messages) !== null && _c !== void 0 ? _c : []);
    }
    /**
     * Get the JSON output from the RetrieveResult.
     *
     * @returns RetrieveCommandResult
     */
    getJson() {
        return this.result;
    }
    /**
     * Displays retrieve results in human format.
     */
    display() {
        if (this.hasStatus(source_deploy_retrieve_1.RequestStatus.InProgress)) {
            // The command timed out
            const commandWaitTime = (0, ts_types_1.getNumber)(this.options, 'waitTime', 1440);
            this.ux.log(messages.getMessage('retrieveTimeout', [commandWaitTime]));
            return;
        }
        if (this.isSuccess()) {
            this.ux.log(`Wrote retrieve zip to ${this.result.zipFilePath}`);
            if (this.options.unzip) {
                const extractPath = (0, path_1.join)(this.options.retrieveTargetDir, (0, path_1.parse)(this.options.zipFileName).name);
                this.ux.log(`Extracted ${this.options.zipFileName} to: ${extractPath}`);
            }
            if (this.options.verbose) {
                const retrievedFiles = (0, resultFormatter_1.toArray)(this.result.fileProperties);
                if (retrievedFiles === null || retrievedFiles === void 0 ? void 0 : retrievedFiles.length) {
                    this.displaySuccesses(retrievedFiles);
                }
            }
            if (this.warnings.length) {
                this.displayWarnings();
            }
        }
        else {
            this.displayErrors();
        }
    }
    hasStatus(status) {
        var _a;
        return ((_a = this.result) === null || _a === void 0 ? void 0 : _a.status) === status;
    }
    displayWarnings() {
        this.ux.styledHeader((0, chalk_1.yellow)(messages.getMessage('retrievedSourceWarningsHeader')));
        const columns = [
            { key: 'fileName', label: 'FILE NAME' },
            { key: 'problem', label: 'PROBLEM' },
        ];
        this.ux.table(this.warnings, { columns });
        this.ux.log();
    }
    displaySuccesses(retrievedFiles) {
        this.sortFileProperties(retrievedFiles);
        this.ux.log('');
        this.ux.styledHeader((0, chalk_1.blue)(`Components Retrieved [${retrievedFiles.length}]`));
        const columns = [
            { key: 'type', label: 'TYPE' },
            { key: 'fileName', label: 'FILE' },
            { key: 'fullName', label: 'NAME' },
            { key: 'id', label: 'ID' },
        ];
        this.ux.table(retrievedFiles, { columns });
    }
    displayErrors() {
        // an invalid packagename retrieval will end up with a message in the `errorMessage` entry
        const errorMessage = (0, ts_types_1.get)(this.result, 'errorMessage');
        if (errorMessage) {
            throw new core_1.SfdxError(errorMessage);
        }
        const unknownMsg = [{ fileName: 'unknown', problem: 'unknown' }];
        const responseMsgs = (0, ts_types_1.get)(this.result, 'messages', unknownMsg);
        const errMsgs = (0, resultFormatter_1.toArray)(responseMsgs);
        const errMsgsForDisplay = errMsgs.reduce((p, c) => `${p}\n${c.fileName}: ${c.problem}`, '');
        this.ux.log(`Retrieve Failed due to: ${errMsgsForDisplay}`);
    }
}
exports.RetrieveResultFormatter = RetrieveResultFormatter;
//# sourceMappingURL=retrieveResultFormatter.js.map